<?php

namespace Intergo\SubscriptionClient;

use Intergo\BaseClient\Responses\BaseResponseInterface;

abstract class AbstractSubscriptionService
{
    protected SubscriptionClient $client;

    protected ?string $token;

    /**
     * AbstractSubscriptionService constructor.
     * https://intergo.atlassian.net/wiki/spaces/ES/pages/2149187587/Communication+between+microservices
     *
     * @param $baseUrl
     * @param $traceId
     * @param $token
     */
    public function __construct($baseUrl, $traceId, $token = null)
    {
        $this->client = new SubscriptionClient($baseUrl, $traceId);
        $this->token = $token;
    }

    /**
     * @param $endpoint
     * @param $bodyParams
     * @param array $urlParams
     * @return BaseResponseInterface
     */
    protected function execute($endpoint, $bodyParams, array $urlParams = [])
    {
        $result = $this->client->execute($endpoint, $urlParams, $bodyParams);
        return $this->response($endpoint, $result);
    }

    /**
     * @param $endpoint
     * @param $bodyParams
     * @param array $urlParams
     * @return BaseResponseInterface
     */
    protected function executeAuth($endpoint, $bodyParams, array $urlParams = [])
    {
        $result = $this->client->executeAuth($endpoint, $this->token, $urlParams, $bodyParams);
        return $this->response($endpoint, $result);
    }

    /**
     * @param string $action
     * @param BaseResponseInterface $result
     * @return BaseResponseInterface
     */
    abstract public function response(string $action, BaseResponseInterface $result);

    /**
     * @return BaseResponseInterface
     */
    public function getPlans()
    {
        return $this->executeAuth(__FUNCTION__, []);
    }

    /**
     * @param $id
     * @return BaseResponseInterface
     */
    public function getPlanById($id)
    {
        return $this->executeAuth(__FUNCTION__, [], compact('id'));
    }

    /**
     * @param $attributes
     * @return BaseResponseInterface
     */
    public function createPlan($attributes)
    {
        extract($attributes);
        unset($attributes);
        return $this->executeAuth(__FUNCTION__, get_defined_vars());
    }

    /**
     * @param $id
     * @param $attributes
     * @return BaseResponseInterface
     */
    public function updatePlan($id, $attributes)
    {
        extract($attributes);
        unset($attributes);
        return $this->executeAuth(__FUNCTION__, get_defined_vars(), compact('id'));
    }

    /**
     * @param $id
     * @return BaseResponseInterface
     */
    public function deletePlan($id)
    {
        return $this->executeAuth(__FUNCTION__, [], compact('id'));
    }

    /**
     * @return BaseResponseInterface
     */
    public function getPlanTypes()
    {
        return $this->executeAuth(__FUNCTION__, []);
    }

    /**
     * @param $id
     * @return BaseResponseInterface
     */
    public function getPlanTypeById($id)
    {
        return $this->executeAuth(__FUNCTION__, [], compact('id'));
    }

    /**
     * @param $attributes
     * @return BaseResponseInterface
     */
    public function createPlanType($attributes)
    {
        extract($attributes);
        unset($attributes);
        return $this->executeAuth(__FUNCTION__, get_defined_vars());
    }

    /**
     * @param $id
     * @param $attributes
     * @return BaseResponseInterface
     */
    public function updatePlanType($id, $attributes)
    {
        extract($attributes);
        unset($attributes);
        return $this->executeAuth(__FUNCTION__, get_defined_vars(), compact('id'));
    }

    /**
     * @param $id
     * @return BaseResponseInterface
     */
    public function deletePlanType($id)
    {
        return $this->executeAuth(__FUNCTION__, [], compact('id'));
    }

    /**
     * @param $queryString
     * @return BaseResponseInterface
     */
    public function getSubscriptionsForManager($queryString)
    {
        if (is_string($queryString)) {
            parse_str($queryString, $queryString);
        }

        extract($queryString);
        unset($queryString);

        return $this->executeAuth(__FUNCTION__, get_defined_vars());
    }

    /**
     * @param $user_id
     * @param $plan_id
     * @param $quantity
     * @return BaseResponseInterface
     */
    public function storeSubscription($user_id, $plan_id, $quantity)
    {
        return $this->execute(__FUNCTION__, get_defined_vars());
    }

    /**
     * @param $id
     * @param $user_id
     * @param $plan_id
     * @param $quantity
     * @return BaseResponseInterface
     */
    public function updateSubscription($id, $user_id = null, $plan_id = null, $quantity = null)
    {
        return $this->execute(__FUNCTION__, compact('user_id', 'plan_id', 'quantity'), compact('id'));
    }

    /**
     * @param $id
     * @return BaseResponseInterface
     */
    public function getSubscriptionById($id)
    {
        return $this->executeAuth(__FUNCTION__, [], compact('id'));
    }

    /**
     * @param $id
     * @return BaseResponseInterface
     */
    public function getSubscriptionBillingLogsBasedOnSubscriptionId($id)
    {
        return $this->executeAuth(__FUNCTION__, [], compact('id'));
    }

    /**
     * @param $ids
     * @return BaseResponseInterface
     */
    public function getPlansByIds($ids)
    {
        return $this->execute(__FUNCTION__, compact('ids'));
    }

    /**
     * @param $ids
     * @return BaseResponseInterface
     */
    public function bulkCancelSubscriptions($ids)
    {
        return $this->execute(__FUNCTION__, compact('ids'));
    }
}